@@ -12,7 +12,7 @@ import math
 from textwrap import dedent
 from pathlib import Path
 
-import tkinter as tk  # Only for typechecking and asserts
+import tkinter as tk
 import tkinter.filedialog
 import tkinter.messagebox
 
@@ -49,7 +49,7 @@ def grid_size(*widgets: 'mixin.MixinWidget') -> 'model.GridSize':
     """
     def maxs(w: 'mixin.MixinWidget') -> typing.Tuple[int, int]:
         info = w.wgrid
-        # logger.debug(f'=> Grid Info: {info}')
+        # logger.debug('=> Grid Info: %s', info)
         if info is None:
             return (-1, -1)  # Not included on a grid yet
         else:
@@ -82,56 +82,21 @@ def configure_grid(master: 'mixin.ContainerWidget',
         column_weights: List of column weights
         row_weights: List of row weights
         kwargs: Extra arguments to the configuration functions
-            `columnconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M8>`_
-            /
-            `rowconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M24>`_.
+            :tk:`columnconfigure <grid.html#M8>` / :tk:`rowconfigure
+            <grid.html#M24>`.
     """
+    # TODO: Remove, use `mixin.ContainerWidget.pgrid`
     if __debug__:
         gw = master.gsize
         gr = model.GridSize(rows=len(row_weights), columns=len(column_weights))
         assert gw == gr, f'{master!r}: Invalid grid size: W::{gw} R::{gr}'
-    assert isinstance(master, (tk.Widget, tk.Tk)), f'{master} is not a valid tkinter.Widget'
+    assert isinstance(master, (tk.Widget, tk.Tk, tk.Toplevel)), f'{master} is not a valid widget'
     for col, w in enumerate(column_weights):
         master.columnconfigure(col, weight=w, **kwargs)  # type: ignore  # Invalid Types
     for row, h in enumerate(row_weights):
         master.rowconfigure(row, weight=h, **kwargs)  # type: ignore  # Invalid Types
 
 
-def configure(master: 'mixin.ContainerWidget',
-              *widgets: 'mixin.MixinWidget',
-              c_rows: bool = False, c_cols: bool = False,
-              **kwargs: typing.Mapping[str, typing.Any]) -> None:
-    '''Configure the rows and columns for the given widgets.
-
-    For widgets that span more than one row or column, the settings are changed
-    for all rows or columns.
-
-    Args:
-        widgets: Widgets to consider.
-        c_rows: Configure the rows
-        c_cols: Configure the columns
-        kwargs: Arguments passed to the configuration functions:
-            `columnconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M8>`_
-            /
-            `rowconfigure <https://www.tcl.tk/man/tcl/TkCmd/grid.html#M24>`_.
-    '''
-    assert isinstance(master, (tk.Widget, tk.Tk)), f'{master} is not a valid tkinter.Widget'
-    for w in widgets:
-        wgrid = w.wgrid
-        if wgrid and c_rows:
-            for r in range(wgrid.row, wgrid.row + wgrid.rowspan):
-                master.rowconfigure(r, **kwargs)  # type: ignore  # Invalid Types
-        if wgrid and c_cols:
-            for c in range(wgrid.column, wgrid.column + wgrid.columnspan):
-                master.columnconfigure(c, **kwargs)  # type: ignore  # Invalid Types
-
-
-# TODO: Remove on v0.18
-def generate_trace(variable: tk.Variable, function: typing.Callable, **kwargs):
-    '''DEPRECATED. See `tkmilan.var.trace`.'''
-    raise NotImplementedError('DEPRECATED')
-
-
 def state_ignore(*toignore: 'mixin.MixinWidget'):
     '''Ignore some subwidgets for container widgets.
 
@@ -192,7 +157,7 @@ def bind_mousewheel(widget, up: typing.Callable, down: typing.Callable, **kwargs
         binding = model.Binding(widget, '<MouseWheel>', wrap_scroll, **kwargs)
         return binding
     else:
-        logger.critical(f'Unsupported system platform: {sys.platform}')
+        logger.critical('Unsupported system platform: %s', sys.platform)
         return NotImplementedError
 
 
@@ -510,3 +475,142 @@ def binding_disable(event=None):
     for further validations.
     '''
     return 'break'
+
+
+def valNumber(string: str) -> typing.Optional[int]:
+    '''Validate a number in any supported base.
+
+    Returns:
+        `None` if not a number, `int` otherwise.
+    '''
+    try:
+        return int(string, base=0)
+    except ValueError:
+        return None
+
+
+def validation_pass(label: str, why: typing.Optional[model.VWhy] = None, **kwargs) -> typing.Optional[bool]:
+    '''Neutral validation function, accepts everything.
+
+    Use this function to validate all states, as the ``fn`` parameter for
+    `model.VSettings`.
+    '''
+    return True
+
+
+def validation_fail(label: str, why: typing.Optional[model.VWhy] = None, **kwargs) -> typing.Optional[bool]:
+    '''Absorvent validation function, denies everything.
+
+    Use this function to invalidate all states, as the ``fn`` parameter for
+    `model.VSettings`.
+    '''
+    return True
+
+
+def state_bindtags(widget: tk.Widget, state: bool, *,
+                   bt_on: typing.Tuple[str, ...],
+                   bt_off: typing.Tuple[str, ...],
+                   ):
+    '''Change the widget state based on ``bindtags`` state.
+
+    This is useful to truly disable all events flowing to a ``widget``, since
+    the ``Tk`` state might keep some events enabled.
+
+    There is no Python documentation, see ``Tk`` :tk:`bindtags <bindtags.htm>`
+    documentation.
+
+    Args:
+        widget: Widget to modify
+        state: Should the widget be enabled (regular state) or disabled (fully
+            dead).
+        bt_on: BindTags when enabled.
+            Mandatory, usually ``widget.bindtags()`` when the widget is
+            created.
+        bt_off: BindTags when disabled.
+            Mandatory, usually ``widget.bindtags()[-2:]`` when the widget is
+            created.
+    '''
+    # This should not be moved to `mixin.MixinWidget`, since it might be used
+    # on non-tkmilan widgets.
+    if state:
+        # Restore BindTags
+        return widget.bindtags(bt_on)
+    else:
+        # Disable BindTags for this widget
+        return widget.bindtags(bt_off)
+
+
+def widget_toolwindow(window: typing.Union[tk.Wm]):
+    '''Change the widget "toolwindow" state.
+
+    This state makes the window present itself as a kind of toolbar window for
+    another window. Can also be used on the `RootWindow`, in which case the
+    window takes similar properties.
+
+    See ``Tk`` :tk:`wm transient <wm.html#M64>` documentation. This is only a
+    window manager hint.
+
+    This is an irreversible action.
+    Depending on the platform, this is defined in several ways:
+
+    - On Windows, set the ``toolwindow`` window attribute, see ``Tk``
+      :tk:`wm attributes toolwindow <wm.html#M13>` documentation.
+    - On Linux, mark the ``type`` window attribute as ``dialog``, see ``Tk``
+      :tk:`wm attributes type dialog <wm.html#M26>` documentation.
+
+    All other platforms are unsupported.
+    '''
+    if sys.platform == 'win32':
+        window.wm_attributes('-toolwindow', True)
+    elif sys.platform.startswith('linux'):
+        window.wm_attributes('-type', 'dialog')
+    else:
+        if __debug__:
+            raise ValueError(f'Unsupported ToolWindow Platform: "{sys.platform}"')
+
+
+def window_center(window: typing.Union[tk.Tk, tk.Toplevel], where: tk.Tk,
+                  deltaW: typing.Optional[int] = None, deltaH: typing.Optional[int] = None,
+                  ) -> bool:
+    '''Center ``window`` on a root window ``where``.
+
+    Considers the sizes of both elements. The ``delta*`` arguments are useful
+    to center the window in a single direction.
+
+    This is only a window manager hint, the geometry might not actually move.
+    This is confirmed in tiling window managers.
+
+    Args:
+        window: The window to move.
+        where: The `RootWindow` to consider as the "center".
+
+        deltaW: Override the calculated delta width.
+        deltaH: Override the calculated delta height.
+
+    Returns:
+        Returns `True` is the window moved to its requested location, `False`
+        otherwise.
+    '''
+    iW = window.winfo_reqwidth()
+    iH = window.winfo_reqheight()
+    # if __debug__:
+    #     logger.debug('Window: %dx%d', iW, iH)
+    #     logger.debug('  Geometry: %s', window.winfo_geometry())
+    oW = where.winfo_width()
+    oH = where.winfo_height()
+    oX = where.winfo_rootx()
+    oY = where.winfo_rooty()
+    # if __debug__:
+    #     logger.debug(' Where: %dx%d @ %d:%d', oW, oH, oX, oY)
+    if deltaW is None:
+        deltaW = (oW - iW) // 2
+    if deltaH is None:
+        deltaH = (oH - iH) // 2
+    # if __debug__:
+    #     logger.debug('Delta:: %dx%d', deltaW, deltaH)
+    x, y = oX + deltaW, oY + deltaH
+    geometry = '+%d+%d' % (x, y)
+    # if __debug__:
+    #     logger.debug('Geometry:: %s', geometry)
+    window.geometry(geometry)
+    return window.geometry() == geometry
